-- Version: 8.5 8.5.0.34

library ieee;
use ieee.std_logic_1164.all;
library fusion;
use fusion.all;

entity Memory is

    port( CLK          : in    std_logic;
          NVM_BUSY_N   : out   std_logic;
          EXT_RAM_CS_N : out   std_logic;
          EXT_RAM_RD_N : out   std_logic;
          EXT_RAM_WR_N : out   std_logic;
          SW0          : in    std_logic;
          MEMRD        : in    std_logic;
          MEMWR        : in    std_logic;
          PSRD         : in    std_logic;
          PSWR         : in    std_logic;
          A16          : out   std_logic;
          NVM_STATUS   : out   std_logic_vector(1 downto 0);
          WD           : in    std_logic_vector(7 downto 0);
          RD           : out   std_logic_vector(7 downto 0);
          ADD          : in    std_logic_vector(17 downto 0);
          EXT_DATA     : inout std_logic_vector(7 downto 0) := (others => 'Z')
        );

end Memory;

architecture DEF_ARCH of Memory is 

  component INT_RAM_8KB
    port( WEN   : in    std_logic := 'U';
          REN   : in    std_logic := 'U';
          RWCLK : in    std_logic := 'U';
          WD    : in    std_logic_vector(7 downto 0) := (others => 'U');
          RD    : out   std_logic_vector(7 downto 0);
          WADDR : in    std_logic_vector(12 downto 0) := (others => 'U');
          RADDR : in    std_logic_vector(12 downto 0) := (others => 'U')
        );
  end component;

  component VCC
    port( Y : out   std_logic
        );
  end component;

  component INV
    port( A : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component GND
    port( Y : out   std_logic
        );
  end component;

  component data_bus_buffer
    port( Trien : in    std_logic := 'U';
          Data  : in    std_logic_vector(7 downto 0) := (others => 'U');
          Y     : out   std_logic_vector(7 downto 0);
          PAD   : inout   std_logic_vector(7 downto 0)
        );
  end component;

  component data_read_mux
    port( Sel0       : in    std_logic := 'U';
          Sel1       : in    std_logic := 'U';
          Data0_port : in    std_logic_vector(7 downto 0) := (others => 'U');
          Data1_port : in    std_logic_vector(7 downto 0) := (others => 'U');
          Data2_port : in    std_logic_vector(7 downto 0) := (others => 'U');
          Result     : out   std_logic_vector(7 downto 0)
        );
  end component;

    signal \CLK\, \data_bus_buffer_0_Y_[7]\, 
        \data_bus_buffer_0_Y_[6]\, \data_bus_buffer_0_Y_[5]\, 
        \data_bus_buffer_0_Y_[4]\, \data_bus_buffer_0_Y_[3]\, 
        \data_bus_buffer_0_Y_[2]\, \data_bus_buffer_0_Y_[1]\, 
        \data_bus_buffer_0_Y_[0]\, \EXT_RAM_WR_N\, 
        \INT_RAM_8KB_0_RD_[7]\, \INT_RAM_8KB_0_RD_[6]\, 
        \INT_RAM_8KB_0_RD_[5]\, \INT_RAM_8KB_0_RD_[4]\, 
        \INT_RAM_8KB_0_RD_[3]\, \INT_RAM_8KB_0_RD_[2]\, 
        \INT_RAM_8KB_0_RD_[1]\, \INT_RAM_8KB_0_RD_[0]\, 
        mem_decode_0_INT_NVM_CS, mem_decode_0_INT_RAM_RD, 
        mem_decode_0_INT_RAM_WR, mem_decode_0_SEL0to0, 
        mem_decode_0_SEL1to1, INV_1_Y, \nvm_if_0_DOUT7to0_[7]\, 
        \nvm_if_0_DOUT7to0_[6]\, \nvm_if_0_DOUT7to0_[5]\, 
        \nvm_if_0_DOUT7to0_[4]\, \nvm_if_0_DOUT7to0_[3]\, 
        \nvm_if_0_DOUT7to0_[2]\, \nvm_if_0_DOUT7to0_[1]\, 
        \nvm_if_0_DOUT7to0_[0]\, GND_net, VCC_net : std_logic;
    signal nc24, nc1, nc8, nc13, nc16, nc19, nc20, nc9, nc22, 
        nc14, nc5, nc21, nc15, nc3, nc10, nc7, nc17, nc4, nc12, 
        nc2, nc23, nc18, nc6, nc11 : std_logic;

begin 

    EXT_RAM_WR_N <= \EXT_RAM_WR_N\;

    mem_decode_0 : entity work.mem_decode
      port map(SW0 => SW0, MEMRD => MEMRD, MEMWR => MEMWR, PSRD
         => PSRD, PSWR => PSWR, A16 => A16, EXT_RAM_WR_N => 
        \EXT_RAM_WR_N\, EXT_RAM_RD_N => EXT_RAM_RD_N, 
        EXT_RAM_CS_N => EXT_RAM_CS_N, INT_RAM_RD => 
        mem_decode_0_INT_RAM_RD, INT_RAM_WR => 
        mem_decode_0_INT_RAM_WR, INT_NVM_CS => 
        mem_decode_0_INT_NVM_CS, UP_ADD(15) => ADD(15), 
        UP_ADD(14) => ADD(14), UP_ADD(13) => ADD(13), SEL(1) => 
        mem_decode_0_SEL1to1, SEL(0) => mem_decode_0_SEL0to0);
    
    INT_RAM_8KB_0 : INT_RAM_8KB
      port map(WEN => mem_decode_0_INT_RAM_WR, REN => 
        mem_decode_0_INT_RAM_RD, RWCLK => \CLK\, WD(7) => WD(7), 
        WD(6) => WD(6), WD(5) => WD(5), WD(4) => WD(4), WD(3) => 
        WD(3), WD(2) => WD(2), WD(1) => WD(1), WD(0) => WD(0), 
        RD(7) => \INT_RAM_8KB_0_RD_[7]\, RD(6) => 
        \INT_RAM_8KB_0_RD_[6]\, RD(5) => \INT_RAM_8KB_0_RD_[5]\, 
        RD(4) => \INT_RAM_8KB_0_RD_[4]\, RD(3) => 
        \INT_RAM_8KB_0_RD_[3]\, RD(2) => \INT_RAM_8KB_0_RD_[2]\, 
        RD(1) => \INT_RAM_8KB_0_RD_[1]\, RD(0) => 
        \INT_RAM_8KB_0_RD_[0]\, WADDR(12) => ADD(12), WADDR(11)
         => ADD(11), WADDR(10) => ADD(10), WADDR(9) => ADD(9), 
        WADDR(8) => ADD(8), WADDR(7) => ADD(7), WADDR(6) => 
        ADD(6), WADDR(5) => ADD(5), WADDR(4) => ADD(4), WADDR(3)
         => ADD(3), WADDR(2) => ADD(2), WADDR(1) => ADD(1), 
        WADDR(0) => ADD(0), RADDR(12) => ADD(12), RADDR(11) => 
        ADD(11), RADDR(10) => ADD(10), RADDR(9) => ADD(9), 
        RADDR(8) => ADD(8), RADDR(7) => ADD(7), RADDR(6) => 
        ADD(6), RADDR(5) => ADD(5), RADDR(4) => ADD(4), RADDR(3)
         => ADD(3), RADDR(2) => ADD(2), RADDR(1) => ADD(1), 
        RADDR(0) => ADD(0));
    
    \VCC\ : VCC
      port map(Y => VCC_net);
    
    INV_0 : INV
      port map(A => CLK, Y => \CLK\);
    
    \GND\ : GND
      port map(Y => GND_net);
    
    nvm_if_0 : entity work.nvm_if
      port map(CLK => \CLK\, RESET => VCC_net, NVM_BUSY => 
        INV_1_Y, AUX_BLOCK => GND_net, DISCARD_PAGE => GND_net, 
        ERASE_PAGE => GND_net, OVERWRITE_PAGE => GND_net, 
        OVERWRITE_PROT => GND_net, PAGELOSS_PROT => GND_net, 
        PROGRAM => GND_net, READ => mem_decode_0_INT_NVM_CS, 
        READ_NEXT => GND_net, LOCK => GND_net, SPARE_PAGE => 
        GND_net, UNPROT_PAGE => GND_net, WRITE => GND_net, 
        PAGE_STATUS => GND_net, ADD(17) => ADD(17), ADD(16) => 
        ADD(16), ADD(15) => ADD(15), ADD(14) => ADD(14), ADD(13)
         => ADD(13), ADD(12) => ADD(12), ADD(11) => ADD(11), 
        ADD(10) => ADD(10), ADD(9) => ADD(9), ADD(8) => ADD(8), 
        ADD(7) => ADD(7), ADD(6) => ADD(6), ADD(5) => ADD(5), 
        ADD(4) => ADD(4), ADD(3) => ADD(3), ADD(2) => ADD(2), 
        ADD(1) => ADD(1), ADD(0) => ADD(0), DATA(31) => GND_net, 
        DATA(30) => GND_net, DATA(29) => GND_net, DATA(28) => 
        GND_net, DATA(27) => GND_net, DATA(26) => GND_net, 
        DATA(25) => GND_net, DATA(24) => GND_net, DATA(23) => 
        GND_net, DATA(22) => GND_net, DATA(21) => GND_net, 
        DATA(20) => GND_net, DATA(19) => GND_net, DATA(18) => 
        GND_net, DATA(17) => GND_net, DATA(16) => GND_net, 
        DATA(15) => GND_net, DATA(14) => GND_net, DATA(13) => 
        GND_net, DATA(12) => GND_net, DATA(11) => GND_net, 
        DATA(10) => GND_net, DATA(9) => GND_net, DATA(8) => 
        GND_net, DATA(7) => GND_net, DATA(6) => GND_net, DATA(5)
         => GND_net, DATA(4) => GND_net, DATA(3) => GND_net, 
        DATA(2) => GND_net, DATA(1) => GND_net, DATA(0) => 
        GND_net, DOUT(31) => nc24, DOUT(30) => nc1, DOUT(29) => 
        nc8, DOUT(28) => nc13, DOUT(27) => nc16, DOUT(26) => nc19, 
        DOUT(25) => nc20, DOUT(24) => nc9, DOUT(23) => nc22, 
        DOUT(22) => nc14, DOUT(21) => nc5, DOUT(20) => nc21, 
        DOUT(19) => nc15, DOUT(18) => nc3, DOUT(17) => nc10, 
        DOUT(16) => nc7, DOUT(15) => nc17, DOUT(14) => nc4, 
        DOUT(13) => nc12, DOUT(12) => nc2, DOUT(11) => nc23, 
        DOUT(10) => nc18, DOUT(9) => nc6, DOUT(8) => nc11, 
        DOUT(7) => \nvm_if_0_DOUT7to0_[7]\, DOUT(6) => 
        \nvm_if_0_DOUT7to0_[6]\, DOUT(5) => 
        \nvm_if_0_DOUT7to0_[5]\, DOUT(4) => 
        \nvm_if_0_DOUT7to0_[4]\, DOUT(3) => 
        \nvm_if_0_DOUT7to0_[3]\, DOUT(2) => 
        \nvm_if_0_DOUT7to0_[2]\, DOUT(1) => 
        \nvm_if_0_DOUT7to0_[1]\, DOUT(0) => 
        \nvm_if_0_DOUT7to0_[0]\, WIDTH(1) => GND_net, WIDTH(0)
         => GND_net, NVM_STATUS(1) => NVM_STATUS(1), 
        NVM_STATUS(0) => NVM_STATUS(0));
    
    data_bus_buffer_0 : data_bus_buffer
      port map(Trien => \EXT_RAM_WR_N\, Data(7) => WD(7), Data(6)
         => WD(6), Data(5) => WD(5), Data(4) => WD(4), Data(3)
         => WD(3), Data(2) => WD(2), Data(1) => WD(1), Data(0)
         => WD(0), Y(7) => \data_bus_buffer_0_Y_[7]\, Y(6) => 
        \data_bus_buffer_0_Y_[6]\, Y(5) => 
        \data_bus_buffer_0_Y_[5]\, Y(4) => 
        \data_bus_buffer_0_Y_[4]\, Y(3) => 
        \data_bus_buffer_0_Y_[3]\, Y(2) => 
        \data_bus_buffer_0_Y_[2]\, Y(1) => 
        \data_bus_buffer_0_Y_[1]\, Y(0) => 
        \data_bus_buffer_0_Y_[0]\, PAD(7) => EXT_DATA(7), PAD(6)
         => EXT_DATA(6), PAD(5) => EXT_DATA(5), PAD(4) => 
        EXT_DATA(4), PAD(3) => EXT_DATA(3), PAD(2) => EXT_DATA(2), 
        PAD(1) => EXT_DATA(1), PAD(0) => EXT_DATA(0));
    
    INV_1 : INV
      port map(A => INV_1_Y, Y => NVM_BUSY_N);
    
    data_read_mux_0 : data_read_mux
      port map(Sel0 => mem_decode_0_SEL0to0, Sel1 => 
        mem_decode_0_SEL1to1, Data0_port(7) => 
        \INT_RAM_8KB_0_RD_[7]\, Data0_port(6) => 
        \INT_RAM_8KB_0_RD_[6]\, Data0_port(5) => 
        \INT_RAM_8KB_0_RD_[5]\, Data0_port(4) => 
        \INT_RAM_8KB_0_RD_[4]\, Data0_port(3) => 
        \INT_RAM_8KB_0_RD_[3]\, Data0_port(2) => 
        \INT_RAM_8KB_0_RD_[2]\, Data0_port(1) => 
        \INT_RAM_8KB_0_RD_[1]\, Data0_port(0) => 
        \INT_RAM_8KB_0_RD_[0]\, Data1_port(7) => 
        \data_bus_buffer_0_Y_[7]\, Data1_port(6) => 
        \data_bus_buffer_0_Y_[6]\, Data1_port(5) => 
        \data_bus_buffer_0_Y_[5]\, Data1_port(4) => 
        \data_bus_buffer_0_Y_[4]\, Data1_port(3) => 
        \data_bus_buffer_0_Y_[3]\, Data1_port(2) => 
        \data_bus_buffer_0_Y_[2]\, Data1_port(1) => 
        \data_bus_buffer_0_Y_[1]\, Data1_port(0) => 
        \data_bus_buffer_0_Y_[0]\, Data2_port(7) => 
        \nvm_if_0_DOUT7to0_[7]\, Data2_port(6) => 
        \nvm_if_0_DOUT7to0_[6]\, Data2_port(5) => 
        \nvm_if_0_DOUT7to0_[5]\, Data2_port(4) => 
        \nvm_if_0_DOUT7to0_[4]\, Data2_port(3) => 
        \nvm_if_0_DOUT7to0_[3]\, Data2_port(2) => 
        \nvm_if_0_DOUT7to0_[2]\, Data2_port(1) => 
        \nvm_if_0_DOUT7to0_[1]\, Data2_port(0) => 
        \nvm_if_0_DOUT7to0_[0]\, Result(7) => RD(7), Result(6)
         => RD(6), Result(5) => RD(5), Result(4) => RD(4), 
        Result(3) => RD(3), Result(2) => RD(2), Result(1) => 
        RD(1), Result(0) => RD(0));
    

end DEF_ARCH; 
